// ZipMaxDlg.h : header file
//

#pragma once


#include <vector>
#include "..\ziparchive\ziparchive.h"


//////////////////////////////////////////////////////////////////////////

class CZipEntry		// used for single zipfile entries
{
	public:

		CZipEntry();


	public:

		CZipFileHeader *m_pHeader;		// headerinformation for current entry
		CString m_strArchiveName;		// zipfilename
		CString m_strEntryName;			// current fileentryname (only used for sorting)


	public:

		// operator used for sorting entries
		bool operator < (const CZipEntry& entry) const;
};



//////////////////////////////////////////////////////////////////////////


class CZipMaxDlg : public CDialog
{
	public:

		CZipMaxDlg(CWnd* pParent = NULL);

		enum { IDD = IDD_ZIPMAX_DIALOG };

	public:

		typedef struct tagThreadParam			// parameters for threadfunction
		{
			DWORD nDoneTime;
			UINT64 nDoneSize;
			UINT64 nRemainingSize;				// remaining size (for statistics only)
			std::vector<CString> *pvecFiles;	// pointer to the filelist
			CString *pstrMainFolder;			// pointer to the zipmax mainfolder name
			CStatic *pStatus1;					// pointer to controls (1x progress + 2x status + 1x list)
			CStatic *pStatus2;
			CProgressCtrl *pProgressCtrl;
			CListCtrl *pListCtrl;
		} THREADPARAM;

	private:

		typedef struct							// Used for sorting the columns in the listview.
		{
			CListCtrl *pListCtrl;
			int nColumn;
			int nSortDirection;
		} SORTINFO;

		std::vector<CString> m_vecFiles;		// the filelist
		CWinThread *m_pThread;					// pointer to started thread
		THREADPARAM m_param;					// thread parameter block

		CListCtrl *m_pListCtrl;					// pointers to the controls
		CStatic *m_pStatus1;
		CStatic *m_pStatus2;
		CProgressCtrl *m_pProgCtrl;
		CButton *m_pButtonPause;

		int m_nSortDirection;
		int m_nLastColumn;
		CImageList m_ImageList;
		CBitmap m_bmStatus[3];
		CBitmap m_bmArrowUp;
		CBitmap m_bmArrowDown;

	private:

		void OnCancel();
		void StartZipping ();
		void GetColumnInfo (int nColumn, LVCOLUMN *pcolumn, CString &str);
		static int CALLBACK CompareProc (LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);


		// static because of threading....

		static int GetThreadPriority();

		static UINT WorkThread(LPVOID pParam);	// the main thread routine

		static BOOL WriteLFH(CStdioFile *pFile, CZipEntry *pZipEntry, DWORD *pOffset, CString *pName);
		static BOOL WriteCFH(CStdioFile *pFile, CZipEntry *pZipEntry, const DWORD nOffset, const CString& strEntryName);
		static BOOL WriteEOZ(CStdioFile *pFile, const DWORD nCentralDirStart, const DWORD nCentralDirSize, const DWORD nEntries);

		static BOOL InternalZipFiles(const CString& strArchiveName, const CString& strSourceFolder, std::vector<CZipFileHeader> *pvecHeaders);
		static BOOL ExternalZipFiles(const CString& strArchiveName, const CString& strSourceFolder, std::vector<CZipFileHeader> *pvecHeaders, const DWORD nExternal);
		static BOOL UnzipAllFiles(const CString& strArchiveName, const CString& strDestinationFolder, std::vector<CZipFileHeader> *pvecHeaders);


		static BOOL GetHeaders(const CString& strArchiveName, std::vector<CZipFileHeader> *pvecHeaders);
		static BOOL BuildNewZip(const CString& strArchiveName, std::vector<CZipEntry> *pvecEntries);
		static BOOL TestNewZip(const CString& strArchiveName);
		static BOOL SafeRename(const CString& strSource);

		static void UpdateList(CListCtrl *pListCtrl, const DWORD nIndex, const CString& s1, const CString& s2, const CString& s3);
		static void UpdateStatus(CStatic *pStatus, const DWORD nIndex = 0, const DWORD nTotalFiles = 0, const DWORD nCurrentTotal = 0, const DWORD nRemainingSecs = 0, const DWORD nDoneSize = 0);
		static void WriteLog(CStdioFile *pFile, const CString& str);


	protected:

		HICON m_hIcon;

	protected:

		virtual void DoDataExchange(CDataExchange* pDX);
		virtual BOOL OnInitDialog();
		afx_msg void OnPaint();
		afx_msg HCURSOR OnQueryDragIcon();
		afx_msg void OnDropFiles(HDROP hDropInfo);
		afx_msg void OnSize(UINT nType, int cx, int cy);
		afx_msg void OnColumnClick (NMHDR* pNMHDR, LRESULT* pResult);
		afx_msg void OnBnClickedButtonPause();

		DECLARE_MESSAGE_MAP()
};
